//C++ header file for Clarion for Windows

/* Check if these work outside of the #pragma
extern "C" int toupper(int);
extern "C" int tolower(int);
extern "C" char *strchr(const char *, int);
*/

typedef unsigned long COLORREF;
typedef unsigned size_t;
typedef struct {
        int     quot;
        int     rem;
}div_t;
typedef struct {
        long    quot;
        long    rem;
}ldiv_t;

extern "C" {                                                //External C linkage for all functions
#pragma save,name(prefix=>""),call(seg_name=>CPPFUNC)

//Clarion Run-Time library: String Stack functions
void            Cla$PopCString(char *, unsigned);
void            Cla$PopPString(char *, unsigned);
void            Cla$PopString(char *, unsigned);
void            Cla$PushCString(char *);
void            Cla$PushString(char *, unsigned);
void            Cla$StackALL(unsigned);
void            Cla$StackCENTER(unsigned);
void            Cla$StackUPPER(void);
void            Cla$StackCLIP(void);
unsigned        Cla$StackLen2(void);
unsigned        Cla$StackLen(void);
int             Cla$StackCompare(void);
int             Cla$StackCompareN(void);
void            Cla$StackConcat(void);
unsigned        Cla$StackINSTRING(unsigned, unsigned);
void            Cla$StackLEFT(unsigned);
void            Cla$StackLOWER(void);
void            Cla$STACKpop(void);
unsigned        Cla$StackNUMERIC(void);
void            Cla$StackPRESS(void);
void            Cla$StackRIGHT(unsigned);
void            Cla$StackSUB(unsigned, unsigned);
unsigned char   Cla$StackVAL(void);
void            Cla$StackUPPER(void);

//Clarion Run-Time
double          Cla$ACOS(double);
void            Cla$ARC(int, int, int, int, int, int);
double          Cla$ASIN(double);
double          Cla$ATAN(double);
void            Cla$BOX(int, int, int, int, COLORREF);
long            Cla$BSHIFT(long, int);
void            Cla$CHORD(int, int, int, int, int, int, COLORREF);
long            Cla$Clock(void);
double          Cla$COS(double);
long            Cla$DATE(unsigned, unsigned, unsigned);
long            Cla$DAY(long);
void            Cla$ELLIPSE(int, int, int, int, COLORREF);
double          Cla$INT(double);
double          Cla$LOG10(double);
double          Cla$LOGE(double);
long            Cla$MONTH(long);
int             Cla$MOUSEX(void);
int             Cla$MOUSEY(void);
unsigned        Cla$NUMERIC(char *, unsigned);
long            Cla$RANDOM(long, long);
double          Cla$ROUND(double, double);
void            Cla$SETCLOCK(long);
void            Cla$SETTODAY(long);
double          Cla$SIN(double);
double          Cla$SQRT(double);
double          Cla$TAN(double);
long            Cla$TODAY(void);
long            Cla$YEAR(long);

//Standard RunTime library functions
//Conversions
double          _atof(const char *);
int             _atoi(const char *);
long            _atol(const char *);
unsigned long   _atoul(const char *);

//Integer Math
int             _abs(int);
long            _labs(long);

//Utility Functions
int             _rand(void);
void            _srand(unsigned);
void *			_malloc(size_t);
void *			_calloc(size_t, size_t);
void *			_realloc(void*, size_t);
void			_free(void*);
void			_exit(int);
void *			_bsearch(const void *, const void *,size_t, size_t, int);
void			_qsort(const void*, size_t, size_t,	int);
div_t           _div(int, int);
ldiv_t			_ldiv(long, long);
char *			_getenv(const char *);

//Low-Level File Manipulation
int             _access(const char *, int);
int             _chmod(const char *, int);
int             _remove(const char *);
int             _rename(const char *, const char *);
void            _fnmerge(char *, const char *, const char *, const char *, const char *);
int             _fnsplit(const char *, char *, char *, char *, char *);
int             _mkdir(const char *);
int             _rmdir(const char *);
int             _getcurdir(int, char *);
int             _chdir(const char *);
int             _getdisk(void);
int             _setdisk(int);

//Time Functions
long			_time(long *__timer); 
long			_clock(void);

//String Functions
char *          _strcat(char *, const char *);
char *          _strcpy(char *, const char *);
int             _strcmp(const char *, const char *);
int             _strequ(const char *, const char *);
unsigned        _strlen(const char *);
char *          _strchr(const char *, int);
unsigned        _strcspn(const char *, const char *);
unsigned        _strspn(const char *, const char *);
char *          _strstr(const char *, const char *);
char *          _strtok(char *, const char *);
char *          _strpbrk(const char *, const char *);
char *          _strrchr(const char *, int);
char *          _strlwr(char *);
char *          _strupr(char *);
char *          _strdup(const char *);
char *          _strrev(char *);
char *          _strncat(char *, const char *, unsigned);
int             _strncmp(const char *, const char *, unsigned);
char *          _strncpy(char *, const char *, unsigned);
int             _strnicmp(const char *, const char *, unsigned);
//
void *		    _memchr(const void *, int, unsigned);
int				_memcmp(const void *, const void *, unsigned);
void*			_memcpy(void *, const void *, unsigned);
void*			_memmove(void *, const void *, unsigned);
void*			_memset(void *, int, unsigned);
//

/* Using macros (see below) at present
int				isalnum(int);
int				isalpha(int);
int				iscntrl(int);
int				isdigit(int);
int				isgraph(int);
int				islower(int);
int				isprint(int);
int				ispunct(int);
int				isspace(int);
int				isupper(int);
int				isxdigit(int); 
//
int	tolower(int c);
int	toupper(int c);
*/

#pragma restore
} //extern "C"

//Macros and other defines
#define isdigit(c) ((c)>='0'&&(c)<='9')
#define isupper(c) ((c)>='A'&&(c)<='Z')
#define islower(c) ((c)>='a'&&(c)<='z')
#define isalpha(c) (isupper(c)||islower(c))
#define isalnum(c) (isalpha(c)||isdigit(c))
#define isprint(c) ((c)>31&&(c)<128)
#define isspace(c) ((c)==' '  || (c)=='\t' ||  \
                    (c)=='\n' || (c)=='\v' ||  \
                    (c)=='\f' || (c)=='\r')
#define toupper(c) (islower(c)?((c)-32):(c))
#define tolower(c) (isupper(c)?((c)+32):(c))

#define min(a,b) ((a)<(b)?(a):(b))
#define max(a,b) ((a)>(b)?(a):(b))
//
#define sqrt    Cla$SQRT
#define acos	Cla$ACOS
#define asin	Cla$ASIN
#define atan	Cla$ATAN
#define bshift  Cla$BSHIFT
#define cos		Cla$COS
#define log10   Cla$LOG10
#define log		Cla$LOGE   //?
#define sin     Cla$SIN
#define tan     Cla$TAN
//
#define abs     _abs
#define labs    _labs
#define div     _div
#define ldiv	_ldiv
#define rand	_rand
#define srand	_srand
//
#define atof    _atof
#define atoi    _atoi
#define atol    _atol
#define atoul   _atoul
//
#define strcat  _strcat
#define strcpy  _strcpy
#define strcmp  _strcmp
#define strequ  _strequ
#define strlen  _strlen
#define strchr  _strchr
#define strcspn _strcspn
#define strspn  _strspn
#define strstr  _strstr
#define strtok  _strtok
#define strpbrk _strpbrk
#define strrchr _strrchr
#define strlwr  _strlwr
#define strupr  _strupr
#define strdup  _strdup
#define strrev  _strrev
#define strncat _strncat
#define strncmp _strncmp
#define strncpy _strncpy
#define strnicmp _strnicmp
#define memchr  _memchr
#define memcmp	_memcmp
#define memcpy	_memcpy
#define memmove	_memmove
#define memset 	_memset
//
#define malloc	_malloc
#define calloc	_calloc
#define realloc	_realloc
#define free	_free
#define exit	_exit
#define getenv  _getenv
//
#define bsearch _bsearch
#define qsort	_qsort
//
#define clock _clock
#define time _time
